/*******************************************************************************
 * (c) Copyright 2016 Microsemi Corporation.  All rights reserved.
 *
 */

#include "hw_platform.h"
#include "drivers/CoreGPIO/core_gpio.h"
#include "drivers/CoreUARTapb/core_uart_apb.h"
#include "drivers/CoreI2C/core_i2c.h"
#include "hal/CortexM1/cortex_nvic.h"
#include "hal.h"
#include "./CMSIS/cortexm1_cfg.h"
#include "./CMSIS/system_cortexm1_cfg.h"

#define DIVISOR 256
#define BAUD_VALUE                  115200

#define HDMI_I2C_MAIN_MAP_ADDR		0x39
#define HDMI_I2C_PACKET_MEM_ADDR	0x70
#define HDMI_I2C_CEC_MEM_ADDR		0x78
#define HDMI_I2C_EDID_MEM_ADDR		0x7E


/******************************************************************************
 * GPIO instance data. UART instance data. Base addresses
 *****************************************************************************/
gpio_instance_t g_gpio_out, g_gpio_in;
UART_instance_t g_uart0, g_uart1;
i2c_instance_t  g_adv7513_ic2, g_adv7511_ic2;

void      LED_update(void);
void	  ADV7513_init(void);
void	  ADV7511_init(void);

int sema_gpio = 0;

__attribute__((__interrupt__)) void IRQ0_IRQHandler( void )
{

	I2C_isr(&g_adv7513_ic2);
	HW_set_32bit_reg(NVIC_CLEAR_PEND_REG, NVIC_IRQ_ADV7513_I2C);
}

__attribute__((__interrupt__)) void IRQ1_IRQHandler( void )
{

	I2C_isr(&g_adv7511_ic2);
	HW_set_32bit_reg(NVIC_CLEAR_PEND_REG, NVIC_IRQ_ADV7511_I2C);
}

__attribute__((__interrupt__)) void IRQ2_IRQHandler( void )
{
	sema_gpio = 1;
	GPIO_clear_all_irq_sources(&g_gpio_in, GPIO_get_irq_sources(&g_gpio_in));
	HW_set_32bit_reg(NVIC_CLEAR_PEND_REG, NVIC_IRQ_GPIO_IN);
}
/*-------------------------------------------------------------------------*//**
 * main() function.
 */
int main()
{
	SystemCoreClockUpdate();

	UART_init(
			&g_uart0,
			UART0_BASE_ADDRESS,
			SYS_M1_CLK_FREQ/((16 * BAUD_VALUE)-1),
			(DATA_8_BITS | NO_PARITY)
	);
	UART_init(
			&g_uart1,
			UART1_BASE_ADDRESS,
			SYS_M1_CLK_FREQ/((16 * BAUD_VALUE)-1),
			(DATA_8_BITS | NO_PARITY)
	);

	I2C_init(
			&g_adv7513_ic2,
			ADV7513_I2C_BASE_ADDR,
			0, // master mode -> don't care
			I2C_PCLK_DIV_256
	);

	I2C_init(
			&g_adv7511_ic2,
			ADV7511_I2C_BASE_ADDR,
			0, // master mode -> don't care
			I2C_PCLK_DIV_256
	);

    /**************************************************************************
     * Initialize the CoreGPIO driver with the base address of the CoreGPIO
     * instance to use and the initial state of the outputs.
     *************************************************************************/
	/* OUTPUTS */
	GPIO_init( &g_gpio_out, COREGPIO_BASE_ADDR, GPIO_APB_32_BITS_BUS );
	/* INPUTS */
	GPIO_init( &g_gpio_in, COREGPIO2_BASE_ADDR, GPIO_APB_32_BITS_BUS );
	GPIO_enable_irq(&g_gpio_in, GPIO_0);
	GPIO_enable_irq(&g_gpio_in, GPIO_1);
	GPIO_enable_irq(&g_gpio_in, GPIO_2);
	GPIO_enable_irq(&g_gpio_in, GPIO_3);

	HW_set_32bit_reg(
			NVIC_ENABLE_REG,
			(NVIC_IRQ_ADV7513_I2C | NVIC_IRQ_ADV7511_I2C | NVIC_IRQ_GPIO_IN)
	);

	ADV7513_init();
	ADV7511_init();

	UART_polled_tx_string(&g_uart0, (uint8_t *)"UART0, Hello World...\n\r");
	UART_polled_tx_string(&g_uart1, (uint8_t *)"UART1, Hello World...\n\r");

	while(1) {
		static uint32_t counter = 0;
		uint8_t key;

    	/* uart0 rx -> uart1 tx */
		if(UART_get_rx(&g_uart0, &key, 1)) {
			UART_send(&g_uart1, &key, 1);
		}
		/* uart1 rx -> uart0 tx */
		if(UART_get_rx(&g_uart1, &key, 1)) {
			UART_send(&g_uart0, &key, 1);
		}
		/* push button hit -> counter reset */
		if(sema_gpio) {
			counter = 0;
			sema_gpio = 0;
		}
		/* counter update */
		if(counter-- ==  0) {
			counter = (SystemCoreClock/DIVISOR);
			LED_update();
		}
     }
     return 0;
}

void LED_update(void)
{
	static uint8_t temp = 1;

	temp = temp^1;
	GPIO_set_output(&g_gpio_out, GPIO_0 ,temp);
	GPIO_set_output(&g_gpio_out, GPIO_1 ,!temp);
	GPIO_set_output(&g_gpio_out, GPIO_2 ,temp);
	GPIO_set_output(&g_gpio_out, GPIO_3 ,!temp);
}

void ADV7513_init(void)
{
	/* *** ADV7513 programming guide section 3 page 14 *** */
	const uint8_t init_script[][2] = {
		/* power up tx */
		{0x41, 0x10},
		/* fixed registers that must be set on power up */
		{0x98, 0x03},
		{0x9A, 0xE0},
		{0x9C, 0x30},
		{0x9D, 0x01},
		{0xA2, 0xA4},
		{0xA3, 0xA4},
		{0xE0, 0xD0},
		{0xF9, 0x00},
		/* audio setup */
		{0x01, 0x00}, // Set N Value(6144)
		{0x02, 0x18}, // Set N Value(6144)
		{0x03, 0x00}, // Set N Value(6144)
		{0x0A, 0x00},
		{0x0B, 0x00},
		{0x0C, 0x00},
		/* video setup */
		{0x15, 0x00}, // [7:4] I2S, [3:0] video input mode
		{0x16, 0x28},
		{0x17, 0x02},
		{0x18, 0x06},
		{0xAF, 0x04},
		{0x97, 0x00}
	};
	i2c_status_t status;
	int i;
	int init_script_length = sizeof(init_script) / sizeof(init_script[0]);

	for(i = 0; i < init_script_length; i++) {
		I2C_write(
			&g_adv7513_ic2,
			HDMI_I2C_MAIN_MAP_ADDR,
			init_script[i],
			2,
			I2C_RELEASE_BUS
		);
		status = I2C_wait_complete(&g_adv7513_ic2, 1000);
	}
	return;
}

void ADV7511_init(void)
{
	const uint8_t init_script[][2] = {
		/* *** ADV7513 programming guide section 3 page 14 *** */
		/* power up tx */
		{0x41, 0x10},
		/* fixed registers that must be set on power up */
		{0x98, 0x03},
		{0x9A, 0xE0},
		{0x9C, 0x30},
		{0x9D, 0x61},
		{0xA2, 0xA4},
		{0xA3, 0xA4},
		{0xE0, 0xD0},
		{0xF9, 0x00},
		/* audio setup */
		{0x01, 0x00}, // Set N Value(6144)
		{0x02, 0x18}, // Set N Value(6144)
		{0x03, 0x00}, // Set N Value(6144)
		{0x0A, 0x00},
		{0x0B, 0x00},
		{0x0C, 0x00},
		/* video setup */
		{0x15, 0x00}, // [7:4] I2S, [3:0] video input mode
		{0x16, 0x28},
		{0x17, 0x02},
		{0x18, 0x06},
		{0xAF, 0x04},
		{0x97, 0x00},
		{0x40, 0x00},
		{0x4C, 0x00}
	};
	i2c_status_t status;
	int i;
	int init_script_length = sizeof(init_script) / sizeof(init_script[0]);
	uint8_t one_byte = 0x04; // MUX SEL HDMI

	I2C_write(
		&g_adv7511_ic2,
		0x70,	// I2C MUX
		&one_byte,
		1,
		I2C_RELEASE_BUS
	);
	status = I2C_wait_complete(&g_adv7511_ic2, 1000);

	for(i = 0; i < init_script_length; i++) {
		I2C_write(
			&g_adv7511_ic2,
			0x39,
			init_script[i],
			2,
			I2C_RELEASE_BUS
		);
		status = I2C_wait_complete(&g_adv7511_ic2, 1000);
	}
	return;
}
